# file: overlap.py
# author: Bob Muller
# date: April 18, 2016
#
# This is a Python program solving the Overlap Graphs problem.
# See: http://rosalind.info/problems/grph/
#
# >>> overlap(filename, N)
#
# where filename is a FASTA file and N is an integer specifying the
# overlap length. The program will generate a file file.graph showing
# the overlap graph.
#
# type id  = string
# type seq = string
# type fasta = (id * seq) list

# seqsMatch : seq * seq * int -> bool
#
# The call seqsMatch(seq1, seq2, n) returns true if n-length suffix
# seq1 is identical to the n-length prefix of seq2.
#
def seqsMatch(seq1, seq2, n):
    return seq1[-n:] == seq2[:n]

# matchAll : seq * fasta * int -> id list
#
# The call matchAll(seq, fasta, n) returns the list of ids of
# sequences in fasta that match seq.
#
def matchAll(seq, fasta, n):
    if fasta == []:
        return []
    else:
        (id, seqX) = fasta[0]
        fasta = fasta[1:]
        almost = matchAll(seq, fasta, n)
        if (seq <> seqX) and seqsMatch(seq, seqX, n):
            return [id] + almost
        else:
            return almost

# readFasta : string -> fasta
#
# The call (readFasta filename) opens filename and returns 
# a list if id * sequence pairs.
#
def readFasta(fasta):
    infile = open(fasta, "r")
    lines = [nextline.strip() for nextline in infile]
    infile.close()
    ans = []
    for i in range(0, len(lines), 2):
        ans.append((lines[i], lines[i + 1]))
    return ans
		     
# overlapGraph : string * int -> (id * id list) list 
#
# The call overlapGraph(fileName, n) opens a fasta file and
# computes the overlap graph if the sequences in the file.
#
def overlapGraph(fileName, n):
    fasta = readFasta(fileName) 
    return map(lambda (id, seq) : (id, matchAll(seq, fasta, n)), fasta)
      
def makeFileName(fname, ext):
    return fname[:fname.index(".")] + ext
		
def writeGraph(graph, file):
    f = open(file, 'w')
    for (id, ids) in graph:
        for i in ids:
            f.write(id + " -> " + i + "\n") 
    f.close()

def go():
  inFileName = "a.fasta"
  n = 3
  graph = overlapGraph(inFileName, n)
  outFileName = makeFileName(inFileName, ".graph")
  writeGraph(graph, outFileName)
        
