# file: complementaryPalindromes.py
# author: Bob Muller
#
# CS1101 Computer Science I
#
# Find the start positions and lengths of complementary palindromes 
# in fasta file sequences where the palindromes have lengths between
# 4 and 12. This is the Location Restriction Sites problem from the
# Rosalind bioinformatics site:
#
# http://rosalind.info/problems/revp/
#

# readFasta : string -> string list
#
# The call (readFasta file) returns a list of strings making up the
# lines in file.
#
# readFasta : string -> string                                                               
#                                                                                            
# The call (readFasta filename) opens filename and returns                                   
# a list if id * sequence pairs.                                                             
#                                                                                            
def readFasta(fasta):
    infile = open(fasta, "r")
    lines = [nextline.strip() for nextline in infile]
    infile.close()

    # Only want the second line containing the DNA sequence.
    #
    return lines[1]

# complements : char -> char -> bool
#
def complements(n1, n2):
    dict = {('A', 'T'):0, ('T', 'A'):0, ('C', 'G'):0, ('G', 'C'):0}
    return (n1, n2) in dict

# reversePalindrome : string -> bool
#
# The call (reversePalindrome seq) returns true of seq is a complementary
# palindrome.
#
def reversePalindrome(seq):
  def repeat(i, j):
      if i >= j:
          return True
      else:
          return complements(seq[i], seq[j]) and repeat(i + 1, j - 1)
  return repeat(0, len(seq) - 1)

# revps : string * int -> int * int list
#
# This is a helper function for complementaryPalindromes (below). 
# NB: eol abbreviates end-of-left, meaning index of rightmost 
# character of left half of a complementary palindrome.
#
def revps(seq, eol):
    def repeat(length, acc):
        start = eol - length / 2
        if start < 0 or (start + length) > len(seq):
            return acc
        else:
            if reversePalindrome(seq[start:start+length]):
                return repeat(length + 2, [(start, length)] + acc)
            else:
                return acc
    return repeat(2, [])

# complementaryPalindromes : string -> int * int list
#
# The call (complementaryPalindromes seq) finds the starting positions 
# and lengths of all complementary palindromic subsequences of seq.  
# NB: Zero-based indexing rather than the 1-based indexing on Rosalind.
#
def complementaryPalindromes(seq):
  def repeat(eol, acc):
    if eol >= len(seq):
        return acc
    else:
        pals = revps(seq, eol)
        return repeat(eol + 1, pals + acc)
  matches = repeat(1, [])
  return filter(lambda (_, length) : length > 3, matches)

# makeOutFile : string -> string
#
# The call makeOutFile(inFile) makes an output file name from an
# input file name.
#
def makeOutFile(fname, ext): return fname[:fname.index(".")] + ext

# writePairs : int * int list -> string -> unit
#
# The call writePairs(pairs, outFile) writes out the starting positions
# and lengths of all of the reverse palindromic sequences.
#
def writePairs(pairs, file):
    f = open(file, 'w')
    for (start, length) in pairs:
        f.write("(" + str(start) + ", " + str(length) + ")\n") 
    f.close()

def go():
  inFile = "a.fas"
  outFile = makeOutFile(inFile, ".restrict")
  dnaSequence = readFasta(inFile)
  pairs = complementaryPalindromes(dnaSequence)
  writePairs(pairs, outFile)

